package ru.dbotthepony.mc.otm.datagen

import net.minecraft.core.HolderSet
import net.minecraft.core.RegistrySetBuilder
import net.minecraft.core.registries.Registries
import net.minecraft.data.worldgen.BootstapContext
import net.minecraft.resources.ResourceKey
import net.minecraft.tags.BiomeTags
import net.minecraft.tags.BlockTags
import net.minecraft.world.level.levelgen.GenerationStep
import net.minecraft.world.level.levelgen.VerticalAnchor
import net.minecraft.world.level.levelgen.feature.ConfiguredFeature
import net.minecraft.world.level.levelgen.feature.Feature
import net.minecraft.world.level.levelgen.feature.configurations.OreConfiguration
import net.minecraft.world.level.levelgen.placement.CountPlacement
import net.minecraft.world.level.levelgen.placement.HeightRangePlacement
import net.minecraft.world.level.levelgen.placement.InSquarePlacement
import net.minecraft.world.level.levelgen.placement.PlacedFeature
import net.minecraft.world.level.levelgen.structure.templatesystem.TagMatchTest
import net.minecraftforge.common.data.DatapackBuiltinEntriesProvider
import net.minecraftforge.common.world.BiomeModifier
import net.minecraftforge.common.world.ForgeBiomeModifiers
import net.minecraftforge.data.event.GatherDataEvent
import net.minecraftforge.registries.ForgeRegistries
import ru.dbotthepony.mc.otm.registry.MBlocks

private object ConfiguredFeatures {
	val TRITANIUM_ORE = key("tritanium_ore")

	private fun key(name: String): ResourceKey<ConfiguredFeature<*, *>> {
		return ResourceKey.create(Registries.CONFIGURED_FEATURE, modLocation(name))
	}
}

fun registerConfiguredFeatures(context: BootstapContext<ConfiguredFeature<*, *>>) {
	val stone = TagMatchTest(BlockTags.STONE_ORE_REPLACEABLES)
	val deepslate = TagMatchTest(BlockTags.DEEPSLATE_ORE_REPLACEABLES)

	val target = listOf(
		OreConfiguration.target(stone, MBlocks.TRITANIUM_ORE.defaultBlockState()),
		OreConfiguration.target(deepslate, MBlocks.DEEPSLATE_TRITANIUM_ORE.defaultBlockState()),
	)

	context.register(ConfiguredFeatures.TRITANIUM_ORE, ConfiguredFeature(Feature.ORE, OreConfiguration(target, 9)))
}

private object PlacedFeatures {
	val NORMAL_TRITANIUM = key("normal_tritanium")
	val DEEP_TRITANIUM = key("deep_tritanium")

	private fun key(name: String): ResourceKey<PlacedFeature> {
		return ResourceKey.create(Registries.PLACED_FEATURE, modLocation(name))
	}
}

fun registerPlacedFeatures(context: BootstapContext<PlacedFeature>) {
	val configured = context.lookup(Registries.CONFIGURED_FEATURE)
	val ore = configured.getOrThrow(ConfiguredFeatures.TRITANIUM_ORE)

	context.register(PlacedFeatures.NORMAL_TRITANIUM, PlacedFeature(
		ore,
		listOf(
			CountPlacement.of(8),
			InSquarePlacement.spread(),
			HeightRangePlacement.triangle(VerticalAnchor.absolute(0), VerticalAnchor.absolute(50))
		)
	))

	context.register(PlacedFeatures.DEEP_TRITANIUM, PlacedFeature(
		ore,
		listOf(
			CountPlacement.of(10),
			InSquarePlacement.spread(),
			HeightRangePlacement.uniform(VerticalAnchor.aboveBottom(8), VerticalAnchor.absolute(0))
		)
	))
}

private object BiomeModifiers {
	val TRITANIUM_ORE = key("tritanium_ore")

	private fun key(name: String): ResourceKey<BiomeModifier> {
		return ResourceKey.create(ForgeRegistries.Keys.BIOME_MODIFIERS, modLocation(name))
	}
}

fun registerBiomeModifiers(context: BootstapContext<BiomeModifier>) {
	val placed = context.lookup(Registries.PLACED_FEATURE)
	val biomes = context.lookup(Registries.BIOME)

	context.register(
		BiomeModifiers.TRITANIUM_ORE,
		ForgeBiomeModifiers.AddFeaturesBiomeModifier(
			biomes.getOrThrow(BiomeTags.IS_OVERWORLD),
			HolderSet.direct(
				placed.getOrThrow(PlacedFeatures.NORMAL_TRITANIUM),
				placed.getOrThrow(PlacedFeatures.DEEP_TRITANIUM)
			),
			GenerationStep.Decoration.UNDERGROUND_ORES
		)
	)
}
